# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/msp/v1alpha1/resource/template.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+nebius/msp/v1alpha1/resource/template.proto\x12\x1cnebius.msp.v1alpha1.resource\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\"\x9e\x01\n\x08Template\x12J\n\x06status\x18\x01 \x01(\x0b\x32,.nebius.msp.v1alpha1.resource.TemplateStatusB\x04\xbaJ\x01\x05R\x06status\x12\x46\n\x04spec\x18\x02 \x01(\x0b\x32*.nebius.msp.v1alpha1.resource.TemplateSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec\"d\n\x0eTemplateStatus\x12R\n\x0epreset_details\x18\x01 \x01(\x0b\x32+.nebius.msp.v1alpha1.resource.PresetDetailsR\rpresetDetails\"\xe7\x01\n\x0cTemplateSpec\x12I\n\tresources\x18\x01 \x01(\x0b\x32+.nebius.msp.v1alpha1.resource.ResourcesSpecR\tresources\x12\x38\n\x05hosts\x18\x02 \x01(\x0b\x32\".nebius.msp.v1alpha1.resource.HostR\x05hosts\x12\x36\n\x04\x64isk\x18\x03 \x01(\x0b\x32\".nebius.msp.v1alpha1.resource.DiskR\x04\x64isk\x12\x1a\n\x04role\x18\x04 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x04role\"S\n\rResourcesSpec\x12\"\n\x08platform\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x08platform\x12\x1e\n\x06preset\x18\x02 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x06preset\"\x84\x01\n\rPresetDetails\x12#\n\tcpu_count\x18\x01 \x01(\x05\x42\x06\xbaH\x03\xc8\x01\x01R\x08\x63puCount\x12\x31\n\x10memory_gibibytes\x18\x02 \x01(\x03\x42\x06\xbaH\x03\xc8\x01\x01R\x0fmemoryGibibytes\x12\x1b\n\tgpu_count\x18\x03 \x01(\x05R\x08gpuCount\"]\n\x05Range\x12\x10\n\x03min\x18\x01 \x01(\x03R\x03min\x12\x10\n\x03max\x18\x02 \x01(\x03R\x03max\x12\x12\n\x04step\x18\x03 \x01(\x03R\x04step\x12\x1c\n\x05value\x18\x04 \x01(\x03\x42\x06\xbaH\x03\xc8\x01\x01R\x05value\"v\n\x04\x44isk\x12\x1a\n\x04type\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x04type\x12R\n\x0esize_gibibytes\x18\x02 \x01(\x0b\x32#.nebius.msp.v1alpha1.resource.RangeB\x06\xbaH\x03\xc8\x01\x01R\rsizeGibibytes\"U\n\x08\x44iskSpec\x12\x1a\n\x04type\x18\x01 \x01(\tB\x06\xbaH\x03\xc8\x01\x01R\x04type\x12-\n\x0esize_gibibytes\x18\x02 \x01(\x03\x42\x06\xbaH\x03\xc8\x01\x01R\rsizeGibibytes\"I\n\x04Host\x12\x41\n\x05\x63ount\x18\x01 \x01(\x0b\x32#.nebius.msp.v1alpha1.resource.RangeB\x06\xbaH\x03\xc8\x01\x01R\x05\x63ount\"(\n\x08HostSpec\x12\x1c\n\x05\x63ount\x18\x01 \x01(\x03\x42\x06\xbaH\x03\xc8\x01\x01R\x05\x63ountBr\n#ai.nebius.pub.msp.v1alpha1.resourceB\rTemplateProtoP\x01Z:github.com/nebius/gosdk/proto/nebius/msp/v1alpha1/resourceb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.msp.v1alpha1.resource.template_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n#ai.nebius.pub.msp.v1alpha1.resourceB\rTemplateProtoP\001Z:github.com/nebius/gosdk/proto/nebius/msp/v1alpha1/resource'
  _TEMPLATE.fields_by_name['status']._options = None
  _TEMPLATE.fields_by_name['status']._serialized_options = b'\272J\001\005'
  _TEMPLATE.fields_by_name['spec']._options = None
  _TEMPLATE.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _TEMPLATESPEC.fields_by_name['role']._options = None
  _TEMPLATESPEC.fields_by_name['role']._serialized_options = b'\272H\003\310\001\001'
  _RESOURCESSPEC.fields_by_name['platform']._options = None
  _RESOURCESSPEC.fields_by_name['platform']._serialized_options = b'\272H\003\310\001\001'
  _RESOURCESSPEC.fields_by_name['preset']._options = None
  _RESOURCESSPEC.fields_by_name['preset']._serialized_options = b'\272H\003\310\001\001'
  _PRESETDETAILS.fields_by_name['cpu_count']._options = None
  _PRESETDETAILS.fields_by_name['cpu_count']._serialized_options = b'\272H\003\310\001\001'
  _PRESETDETAILS.fields_by_name['memory_gibibytes']._options = None
  _PRESETDETAILS.fields_by_name['memory_gibibytes']._serialized_options = b'\272H\003\310\001\001'
  _RANGE.fields_by_name['value']._options = None
  _RANGE.fields_by_name['value']._serialized_options = b'\272H\003\310\001\001'
  _DISK.fields_by_name['type']._options = None
  _DISK.fields_by_name['type']._serialized_options = b'\272H\003\310\001\001'
  _DISK.fields_by_name['size_gibibytes']._options = None
  _DISK.fields_by_name['size_gibibytes']._serialized_options = b'\272H\003\310\001\001'
  _DISKSPEC.fields_by_name['type']._options = None
  _DISKSPEC.fields_by_name['type']._serialized_options = b'\272H\003\310\001\001'
  _DISKSPEC.fields_by_name['size_gibibytes']._options = None
  _DISKSPEC.fields_by_name['size_gibibytes']._serialized_options = b'\272H\003\310\001\001'
  _HOST.fields_by_name['count']._options = None
  _HOST.fields_by_name['count']._serialized_options = b'\272H\003\310\001\001'
  _HOSTSPEC.fields_by_name['count']._options = None
  _HOSTSPEC.fields_by_name['count']._serialized_options = b'\272H\003\310\001\001'
  _globals['_TEMPLATE']._serialized_start=133
  _globals['_TEMPLATE']._serialized_end=291
  _globals['_TEMPLATESTATUS']._serialized_start=293
  _globals['_TEMPLATESTATUS']._serialized_end=393
  _globals['_TEMPLATESPEC']._serialized_start=396
  _globals['_TEMPLATESPEC']._serialized_end=627
  _globals['_RESOURCESSPEC']._serialized_start=629
  _globals['_RESOURCESSPEC']._serialized_end=712
  _globals['_PRESETDETAILS']._serialized_start=715
  _globals['_PRESETDETAILS']._serialized_end=847
  _globals['_RANGE']._serialized_start=849
  _globals['_RANGE']._serialized_end=942
  _globals['_DISK']._serialized_start=944
  _globals['_DISK']._serialized_end=1062
  _globals['_DISKSPEC']._serialized_start=1064
  _globals['_DISKSPEC']._serialized_end=1149
  _globals['_HOST']._serialized_start=1151
  _globals['_HOST']._serialized_end=1224
  _globals['_HOSTSPEC']._serialized_start=1226
  _globals['_HOSTSPEC']._serialized_end=1266
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/msp/v1alpha1/cluster.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!nebius/msp/v1alpha1/cluster.proto\x12\x13nebius.msp.v1alpha1\"\xfd\x03\n\rClusterStatus\x12>\n\x05phase\x18\x01 \x01(\x0e\x32(.nebius.msp.v1alpha1.ClusterStatus.PhaseR\x05phase\x12>\n\x05state\x18\x02 \x01(\x0e\x32(.nebius.msp.v1alpha1.ClusterStatus.StateR\x05state\x12 \n\x0breconciling\x18\x64 \x01(\x08R\x0breconciling\"\xc3\x01\n\x05Phase\x12\x15\n\x11PHASE_UNSPECIFIED\x10\x00\x12\x16\n\x12PHASE_PROVISIONING\x10\x01\x12\x11\n\rPHASE_RUNNING\x10\x02\x12\x12\n\x0ePHASE_UPDATING\x10\x03\x12\x12\n\x0ePHASE_DELETING\x10\x04\x12\x15\n\rPHASE_DELETED\x10\x05\x1a\x02\x08\x01\x12\x11\n\rPHASE_PURGING\x10\x06\x12\x12\n\x0ePHASE_STOPPING\x10\x07\x12\x12\n\x0ePHASE_RESUMING\x10\x08\"\x83\x01\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\x15\n\x11STATE_IN_PROGRESS\x10\x01\x12\x12\n\x0eSTATE_FINISHED\x10\x02\x12\x0f\n\x0bSTATE_ERROR\x10\x03\x12\x12\n\x0eSTATE_DEGRADED\x10\x04\x12\x13\n\x0fSTATE_SCHEDULED\x10\x05\x42_\n\x1a\x61i.nebius.pub.msp.v1alpha1B\x0c\x43lusterProtoP\x01Z1github.com/nebius/gosdk/proto/nebius/msp/v1alpha1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.msp.v1alpha1.cluster_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\032ai.nebius.pub.msp.v1alpha1B\014ClusterProtoP\001Z1github.com/nebius/gosdk/proto/nebius/msp/v1alpha1'
  _CLUSTERSTATUS_PHASE.values_by_name["PHASE_DELETED"]._options = None
  _CLUSTERSTATUS_PHASE.values_by_name["PHASE_DELETED"]._serialized_options = b'\010\001'
  _globals['_CLUSTERSTATUS']._serialized_start=59
  _globals['_CLUSTERSTATUS']._serialized_end=568
  _globals['_CLUSTERSTATUS_PHASE']._serialized_start=239
  _globals['_CLUSTERSTATUS_PHASE']._serialized_end=434
  _globals['_CLUSTERSTATUS_STATE']._serialized_start=437
  _globals['_CLUSTERSTATUS_STATE']._serialized_end=568
# @@protoc_insertion_point(module_scope)



"""Convenience bearer for federated credentials.

This module exposes :class:`FederatedCredentialsBearer`, a convenience
wrapper that composes the low-level federated credentials reader/token
requester implementations with the exchangeable and renewable bearers
to provide a ready-to-use bearer producing short-lived access tokens.

The class accepts one of several inputs for ``federated_credentials``:

- a
    :class:`nebius.base.service_account.federated_credentials.FederatedCredentialsBearer`
    (a reader-like object),
- a
    :class:`nebius.base.service_account.federated_credentials.FederatedCredentialsTokenRequester`
    (an object that can construct exchange requests), or
- a string path pointing to a file understood by
  :class:`nebius.base.service_account.federated_credentials.FileFederatedCredentials`.

The resulting bearer performs the token exchange via
:class:`nebius.aio.token.exchangeable.Bearer` and adds background
refresh via :class:`nebius.aio.token.renewable.Bearer`. When the
underlying credentials originate from a file the resulting bearer is
wrapped in a :class:`nebius.aio.token.token.NamedBearer` to provide a
stable diagnostic name.

Example
-------

Using a file path::

    from nebius.aio.token.federated_credentials import FederatedCredentialsBearer
    bearer = FederatedCredentialsBearer(
        "/path/to/fed-credentials.json",
        service_account_id="sa-123",
    )
    token = await bearer.receiver().fetch()

Using an existing reader/token requester::

    reader = SomeReader(...)
    bearer = FederatedCredentialsBearer(reader, service_account_id="sa-123")
"""

from datetime import timedelta

from nebius.aio.abc import ClientChannelInterface
from nebius.aio.token.deferred_channel import DeferredChannel
from nebius.aio.token.exchangeable import Bearer as ExchangeableBearer
from nebius.aio.token.renewable import Bearer as RenewableBearer
from nebius.aio.token.token import Bearer as ParentBearer
from nebius.aio.token.token import NamedBearer, Receiver
from nebius.base.service_account.federated_credentials import (
    FederatedCredentialsBearer as FederatedCredentialsReader,
)
from nebius.base.service_account.federated_credentials import (
    FederatedCredentialsTokenRequester,
    FileFederatedCredentials,
)


class FederatedCredentialsBearer(ParentBearer):
    """Bearer that exchanges federated credentials for access tokens.

    The class composes an :class:`ExchangeableBearer` (performs the
    token exchange) wrapped by :class:`RenewableBearer` (background
    refresh). It is a convenience wrapper for the common case where a
    consumer has federated credentials stored in a file or provided via
    a reader.

    If the federated credentials originate from a file, the resulting
    bearer is wrapped in a :class:`NamedBearer` to provide a stable cacheable name.

    Parameters are the same as for the underlying components and are
    passed through accordingly. See the examples in the module
    docstring for common usage patterns.

    :param federated_credentials: Either a reader, a token requester
        or a string path pointing to a file containing federated
        credentials. When a string is provided it is interpreted via
        :class:`FileFederatedCredentials`.
    :param service_account_id: Required when ``federated_credentials``
        is a reader or a string; identifies the target service account for the
        exchange.
    :param channel: Optional gRPC channel used for token exchange or
        a :class:`DeferredChannel` that resolves to a channel later.
    :param max_retries: Maximum per-request retry attempts.
    :param lifetime_safe_fraction: Fraction of token lifetime before
        triggering refresh.
    :param initial_retry_timeout: Initial retry backoff.
    :param max_retry_timeout: Maximum retry backoff.
    :param retry_timeout_exponent: Exponential backoff base.
    :param refresh_request_timeout: Timeout for a single refresh
        request.

    Example
    -------

    Construct a bearer and use it to initialize the SDK::

        from asyncio import Future
        from nebius.sdk import SDK
        from nebius.aio.token.federated_credentials import FederatedCredentialsBearer

        # Create a future for the channel that will be resolved with the SDK
        channel_future = Future()

        sdk = SDK(credentials=FederatedCredentialsBearer(
            federated_credentials="/path/to/fed-credentials.json",
            service_account_id="your-service-account-id",
            channel=channel_future,
        ))

        # Resolve the future with the newly created SDK
        channel_future.set_result(sdk)
    """

    def __init__(
        self,
        federated_credentials: (
            FederatedCredentialsTokenRequester | FederatedCredentialsReader | str
        ),
        service_account_id: str | None = None,
        channel: ClientChannelInterface | DeferredChannel | None = None,
        max_retries: int = 2,
        lifetime_safe_fraction: float = 0.9,
        initial_retry_timeout: timedelta = timedelta(seconds=1),
        max_retry_timeout: timedelta = timedelta(minutes=1),
        retry_timeout_exponent: float = 1.5,
        refresh_request_timeout: timedelta = timedelta(seconds=5),
    ) -> None:
        """Create a federated credentials backed bearer."""
        if isinstance(federated_credentials, str):
            federated_credentials = FileFederatedCredentials(federated_credentials)
        if isinstance(federated_credentials, FederatedCredentialsReader):
            if not isinstance(service_account_id, str):
                raise TypeError(
                    "Service account ID must be provided as a string when "
                    "federated_credentials is a string."
                )
            federated_credentials = FederatedCredentialsTokenRequester(
                service_account_id=service_account_id,
                credentials=federated_credentials,
            )

        if not isinstance(federated_credentials, FederatedCredentialsTokenRequester):  # type: ignore[unused-ignore]
            raise TypeError(
                "federated_credentials must be FederatedCredentialsTokenRequester, "
                "FederatedCredentialsBearer or string"
                f", got {type(federated_credentials)}"
            )

        self._exchangeable = ExchangeableBearer(
            federated_credentials,
            channel=channel,
            max_retries=max_retries,
        )
        self._source: ParentBearer = RenewableBearer(
            self._exchangeable,
            max_retries=max_retries,
            lifetime_safe_fraction=lifetime_safe_fraction,
            initial_retry_timeout=initial_retry_timeout,
            max_retry_timeout=max_retry_timeout,
            retry_timeout_exponent=retry_timeout_exponent,
            refresh_request_timeout=refresh_request_timeout,
        )

        if isinstance(federated_credentials.credentials, FileFederatedCredentials):
            self._source = NamedBearer(
                self._source,
                f"federated-credentials/{federated_credentials.service_account_id}"
                f"/{federated_credentials.credentials.file_path}",
            )

    def set_channel(self, channel: ClientChannelInterface) -> None:
        """Attach a concrete gRPC channel to the underlying exchangeable.

        This is a simple convenience method that forwards the channel to
        the embedded :class:`ExchangeableBearer`.
        """
        self._exchangeable.set_channel(channel)

    @property
    def wrapped(self) -> "ParentBearer|None":
        """Return the outermost wrapped bearer (typically the renewable source).

        :returns: The wrapped :class:`ParentBearer` used by this convenience wrapper.
        """
        return self._source

    def receiver(self) -> "Receiver":
        """Return a per-request receiver constructed from the underlying renewable
        bearer.

        :returns: A :class:`Receiver` from the underlying renewable bearer.
        """
        return self._source.receiver()



from typing_extensions import Unpack

from nebius.aio.channel import Channel
from nebius.aio.request import Request
from nebius.aio.request_kwargs import RequestKwargs
from nebius.api.nebius.iam.v1 import (
    GetProfileRequest,
    GetProfileResponse,
    ProfileServiceClient,
)


class SDK(Channel):
    """High-level SDK facade providing convenience helpers for Nebius services.

    The SDK class is a thin convenience wrapper around a gRPC channel that
    exposes higher-level helpers for common operations such as retrieving the
    current authenticated profile. It inherits from :class:`Channel`
    and therefore also exposes the channel-level behaviors and helpers
    (resolution, pooling, credential wiring, sync/async helpers).

    Quick start -- initialization
    =============================

    Common ways to construct the SDK:

    - From an IAM token in the environment (default behavior)::

        sdk = SDK()

    - With an explicit token string or static bearer::

        sdk = SDK(credentials="MY_IAM_TOKEN")
        # or
        sdk = SDK(credentials=Bearer("MY_IAM_TOKEN"))

    - From an env-backed token provider::

        from nebius.aio.token.static import EnvBearer
        sdk = SDK(credentials=EnvBearer("NEBIUS_IAM_TOKEN"))

    - From the CLI config reader (reads endpoints/profile like the CLI)::

        from nebius.aio.cli_config import Config
        sdk = SDK(config_reader=Config())

    - Service account private key or credentials file::

        sdk = SDK(service_account_private_key_file_name="private.pem",
                service_account_public_key_id="pub-id",
                service_account_id="service-account-id")
        # or
        sdk = SDK(credentials_file_name="path/to/credentials.json")

    Async vs sync usage and lifecycle
    ---------------------------------

    The SDK is designed for asyncio. Prefer the async context manager which
    ensures graceful shutdown of background tasks::

        async with SDK(...) as sdk:
            resp = await sdk.whoami()

    Synchronous usage is supported but riskier. It may raise
    :class:`nebius.aio.channel.LoopError` if used inside an active loop. For
    sync use::

          sdk = SDK(...)
          try:
              resp = sdk.whoami().wait()
          finally:
              sdk.sync_close()

    If you provide a separate event loop at construction you can safely call
    synchronous helpers from threads; otherwise avoid mixing sync calls with
    a running event loop.

    Authentication and ``auth_timeout``
    -----------------------------------

    - Many calls accept an ``auth_timeout`` parameter which bounds the total
      time spent acquiring or renewing credentials plus the enclosed request.
    - Default: 15 minutes (900 seconds). Pass ``auth_timeout=None`` to disable
      the bound (be cautious -- this can hang indefinitely if auth cannot
      complete).
    - ``auth_options`` may be provided to control renewal behavior (for example
      to make token renewal synchronous or to surface renewal errors as request
      errors).

    Timeouts and retries (summary)
    ------------------------------

    - The SDK uses two timeout levels:
        * overall request timeout (deadline for entire request including retries),
        * per-retry timeout (deadline for each individual retry attempt).
    - Defaults used by the SDK (unless overridden by a call):
        * overall request timeout: 60s
        * per-retry timeout: 20s (derived from 60s / 3 retries)
    - Set ``timeout=None`` to disable the request deadline.
    - Configure ``retries`` and ``per_retry_timeout`` per-call when needed.

    Parent ID auto-population
    -------------------------

    - Some methods (``list``, ``get_by_name``, ``get``, many others) may have
      a ``parent_id`` automatically populated from CLI
      :class:`nebius.aio.cli_config.Config` or the SDK ``parent_id`` provided at
      initialization.
    - To disable automatic parent population while retaining CLI config use the
      Config option ``no_parent_id=True``.

    Operations
    ----------

    - Long-running operations returned by service ``create``/``update`` or other calls
      are represented by an :class:`nebius.aio.operation.Operation` wrapper that can be
      awaited until completion. You can also list operations via the source service's
      ``operation_service()`` helper.
    - The ``Operation`` wrapper provides convenience helpers like ``.wait()`` and
      ``.resource_id``.

    Request metadata and debugging
    ------------------------------

    - Service methods return :class:`Request` objects. These provide access to
      additional metadata (request id, trace id) and can be awaited or waited on
      synchronously.
    - Example::

        request = sdk.whoami()   # does not await immediately
        resp = await request
        request_id = await request.request_id()
        trace_id = await request.trace_id()

    Error handling and :class:`nebius.aio.service_error.RequestError`
    -----------------------------------------------------------------

    - Server-created errors derive from :class:`nebius.aio.service_error.RequestError`
      and include structured information. Catch and inspect ``err.status`` for
      the server-provided details.

    User-agent customization
    ------------------------

    - Add custom user-agent parts via ``options`` (``grpc.primary_user_agent``)
      or the ``user_agent_prefix`` parameter when constructing the SDK. The SDK
      composes the final user-agent from these pieces and the internal SDK
      version string.

    See also
    --------

    - Full usage examples and deeper explanations are available in the
      project README and the API reference documentation.
    """

    def whoami(
        self,
        **kwargs: Unpack[RequestKwargs],
    ) -> Request[GetProfileRequest, GetProfileResponse]:
        """Return a request that fetches the profile for the current credentials.

        This is a convenience wrapper around the generated
        :class:`ProfileServiceClient.get` method.

        Request arguments may be provided as keyword arguments.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.

        :return: A :class:`Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`Request` of
            :class:`GetProfileResponse`
        """

        client = ProfileServiceClient(self)
        return client.get(
            GetProfileRequest(),
            **kwargs,
        )



import os
import platform


def get_system_certificates() -> str:
    system_cert_path = None

    if platform.system() == "Linux":
        # Common Linux trust store paths
        possible_paths = [
            "/etc/ssl/certs/ca-certificates.crt",  # Debian/Ubuntu
            "/etc/pki/tls/certs/ca-bundle.crt",  # RHEL/CentOS
            "/etc/ssl/ca-bundle.pem",  # OpenSUSE
        ]
        for path in possible_paths:
            if os.path.exists(path):
                system_cert_path = path
                break
    elif platform.system() == "Darwin":  # macOS
        # macOS requires fetching certs via security CLI
        possible_paths = [
            "/opt/homebrew/etc/openssl@3/cert.pem",  # New
            "/usr/local/etc/openssl@1.1/cert.pem",  # Deprecated
        ]
        for path in possible_paths:
            if os.path.exists(path):
                system_cert_path = path
                break
    elif platform.system() == "Windows":
        # On Windows, use certifi-win32 or fetch manually
        try:
            import certifi_win32  # type: ignore[unused-ignore,import-not-found]

            system_cert_path = str(certifi_win32.wincerts.where())  # type: ignore[unused-ignore]
        except ImportError:
            raise RuntimeError("Install 'certifi-win32' to access Windows certificates")

    if not system_cert_path or not os.path.exists(system_cert_path):
        raise RuntimeError("System certificate bundle not found.")
    return system_cert_path



# 
# Generated by the nebius.base.protos.compiler.  DO NOT EDIT!
# 

"""Auto-generated Nebius SDK package for ``nebius.maintenance.v1alpha1``"""

import builtins as builtins
import collections.abc as abc
import datetime as datetime
import google.protobuf.descriptor as descriptor_1
import google.protobuf.message as message_1
import google.protobuf.timestamp_pb2 as timestamp_pb2
import nebius.aio.client as client
import nebius.aio.request as request
import nebius.aio.request_kwargs as request_kwargs
import nebius.api.nebius.common.v1 as v1_1
import nebius.api.nebius.common.v1.metadata_pb2 as metadata_pb2
import nebius.api.nebius.maintenance.v1alpha1.maintenance_pb2 as maintenance_pb2
import nebius.api.nebius.maintenance.v1alpha1.maintenance_service_pb2 as maintenance_service_pb2
import nebius.base.protos.descriptor as descriptor
import nebius.base.protos.pb_classes as pb_classes
import nebius.base.protos.pb_enum as pb_enum
import nebius.base.protos.unset as unset
import nebius.base.protos.well_known as well_known_1
import typing_extensions as typing_extensions
#@ local imports here @#

# file: nebius/maintenance/v1alpha1/maintenance.proto
class State(pb_enum.Enum):
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.EnumDescriptor](".nebius.maintenance.v1alpha1.State",maintenance_pb2.DESCRIPTOR,descriptor_1.EnumDescriptor)
    STATE_UNSPECIFIED = 0
    STATE_SCHEDULED = 1
    """
    Maintenance operation is scheduled to be run in the future.
    """
    
    STATE_IN_PROGRESS = 2
    """
    Maintenance operation is currently in progress.
    """
    
    STATE_FINISHED = 3
    """
    Maintenance operation has been successfully completed.
    """
    
    STATE_ERROR = 4
    """
    Maintenance operation has encountered an error and could not be completed.
    """
    
    STATE_CANCELLED = 5
    """
    Maintenance operation has been cancelled by the system.
    """
    

class Maintenance(pb_classes.Message):
    """
    Maintenance operations are initiated by Nebius Cloud infrastructure.
    They are used to perform scheduled or urgent operations on Nebius Cloud resources,
    such as Managed Services or Standalone Applications, that are necessary for
    maintaining the service in an operable state. These operations run automatically,
    without user intervention.
    """
    
    __PB2_CLASS__ = maintenance_pb2.Maintenance
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.maintenance.v1alpha1.Maintenance",maintenance_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "MaintenanceSpec|maintenance_pb2.MaintenanceSpec|None|unset.UnsetType" = unset.Unset,
        status: "MaintenanceStatus|maintenance_pb2.MaintenanceStatus|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
        if not isinstance(status, unset.UnsetType):
            self.status = status
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
            "status",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        """
        The metadata for the maintenance operation resource.
        """
        
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "MaintenanceSpec":
        """
        Specifications for the maintenance operation.
        """
        
        return super()._get_field("spec", explicit_presence=False,
        wrap=MaintenanceSpec,
        )
    @spec.setter
    def spec(self, value: "MaintenanceSpec|maintenance_pb2.MaintenanceSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    @builtins.property
    def status(self) -> "MaintenanceStatus":
        """
        Current status of the maintenance operation.
        """
        
        return super()._get_field("status", explicit_presence=False,
        wrap=MaintenanceStatus,
        )
    @status.setter
    def status(self, value: "MaintenanceStatus|maintenance_pb2.MaintenanceStatus|None") -> None:
        return super()._set_field("status",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
        "status":"status",
    }
    
class MaintenanceSpec(pb_classes.Message):
    __PB2_CLASS__ = maintenance_pb2.MaintenanceSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.maintenance.v1alpha1.MaintenanceSpec",maintenance_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
        "scheduled_at": well_known_1.ts_mask,
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        scheduled_at: "timestamp_pb2.Timestamp|datetime.datetime|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(scheduled_at, unset.UnsetType):
            self.scheduled_at = scheduled_at
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "scheduled_at",
        ]
    
    @builtins.property
    def scheduled_at(self) -> "datetime.datetime":
        """
        Timestamp indicating when the maintenance operation is scheduled to run.
        """
        
        return super()._get_field("scheduled_at", explicit_presence=False,
        wrap=well_known_1.from_timestamp
        )
    @scheduled_at.setter
    def scheduled_at(self, value: "timestamp_pb2.Timestamp|datetime.datetime|None") -> None:
        return super()._set_field("scheduled_at",value,explicit_presence=False,
        unwrap=well_known_1.to_timestamp
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "scheduled_at":"scheduled_at",
    }
    
class MaintenanceStatus(pb_classes.Message):
    __PB2_CLASS__ = maintenance_pb2.MaintenanceStatus
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.maintenance.v1alpha1.MaintenanceStatus",maintenance_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
        "started_at": well_known_1.ts_mask,
        "finished_at": well_known_1.ts_mask,
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        affected_resources: "abc.Iterable[Resource]|None|unset.UnsetType" = unset.Unset,
        started_at: "timestamp_pb2.Timestamp|datetime.datetime|None|unset.UnsetType" = unset.Unset,
        finished_at: "timestamp_pb2.Timestamp|datetime.datetime|None|unset.UnsetType" = unset.Unset,
        state: "State|maintenance_pb2.State|None|unset.UnsetType" = unset.Unset,
        reschedulable: "builtins.bool|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(affected_resources, unset.UnsetType):
            self.affected_resources = affected_resources
        if not isinstance(started_at, unset.UnsetType):
            self.started_at = started_at
        if not isinstance(finished_at, unset.UnsetType):
            self.finished_at = finished_at
        if not isinstance(state, unset.UnsetType):
            self.state = state
        if not isinstance(reschedulable, unset.UnsetType):
            self.reschedulable = reschedulable
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "affected_resources",
            "started_at",
            "finished_at",
            "state",
            "reschedulable",
        ]
    
    @builtins.property
    def affected_resources(self) -> "abc.MutableSequence[Resource]":
        """
        List of resources that are affected by this maintenance operation.
        """
        
        return super()._get_field("affected_resources", explicit_presence=False,
        wrap=pb_classes.Repeated.with_wrap(Resource,None,None),
        )
    @affected_resources.setter
    def affected_resources(self, value: "abc.Iterable[Resource]|None") -> None:
        return super()._set_field("affected_resources",value,explicit_presence=False,
        )
    
    @builtins.property
    def started_at(self) -> "datetime.datetime":
        """
        Timestamp indicating when the maintenance operation started.
        """
        
        return super()._get_field("started_at", explicit_presence=False,
        wrap=well_known_1.from_timestamp
        )
    @started_at.setter
    def started_at(self, value: "timestamp_pb2.Timestamp|datetime.datetime|None") -> None:
        return super()._set_field("started_at",value,explicit_presence=False,
        unwrap=well_known_1.to_timestamp
        )
    
    @builtins.property
    def finished_at(self) -> "datetime.datetime":
        """
        Timestamp indicating when the maintenance operation finished.
        """
        
        return super()._get_field("finished_at", explicit_presence=False,
        wrap=well_known_1.from_timestamp
        )
    @finished_at.setter
    def finished_at(self, value: "timestamp_pb2.Timestamp|datetime.datetime|None") -> None:
        return super()._set_field("finished_at",value,explicit_presence=False,
        unwrap=well_known_1.to_timestamp
        )
    
    @builtins.property
    def state(self) -> "State":
        """
        Current state of the maintenance operation.
        """
        
        return super()._get_field("state", explicit_presence=False,
        wrap=State,
        )
    @state.setter
    def state(self, value: "State|maintenance_pb2.State|None") -> None:
        return super()._set_field("state",value,explicit_presence=False,
        )
    
    @builtins.property
    def reschedulable(self) -> "builtins.bool":
        """
        Indicates whether the maintenance operation's scheduled_at time can be changed.
        """
        
        return super()._get_field("reschedulable", explicit_presence=False,
        )
    @reschedulable.setter
    def reschedulable(self, value: "builtins.bool|None") -> None:
        return super()._set_field("reschedulable",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "affected_resources":"affected_resources",
        "started_at":"started_at",
        "finished_at":"finished_at",
        "state":"state",
        "reschedulable":"reschedulable",
    }
    
class Resource(pb_classes.Message):
    __PB2_CLASS__ = maintenance_pb2.Resource
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.maintenance.v1alpha1.Resource",maintenance_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        parent_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
        if not isinstance(parent_id, unset.UnsetType):
            self.parent_id = parent_id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
            "parent_id",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        Unique identifier for the resource.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    @builtins.property
    def parent_id(self) -> "builtins.str":
        """
        IAM container of the resource.
        """
        
        return super()._get_field("parent_id", explicit_presence=False,
        )
    @parent_id.setter
    def parent_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("parent_id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
        "parent_id":"parent_id",
    }
    
# file: nebius/maintenance/v1alpha1/maintenance_service.proto
class ListMaintenancesRequest(pb_classes.Message):
    __PB2_CLASS__ = maintenance_service_pb2.ListMaintenancesRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.maintenance.v1alpha1.ListMaintenancesRequest",maintenance_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        parent_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(parent_id, unset.UnsetType):
            self.parent_id = parent_id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "parent_id",
        ]
    
    @builtins.property
    def parent_id(self) -> "builtins.str":
        """
        Identifier of IAM container to list maintenance operations from.
        """
        
        return super()._get_field("parent_id", explicit_presence=False,
        )
    @parent_id.setter
    def parent_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("parent_id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "parent_id":"parent_id",
    }
    
class ListMaintenancesResponse(pb_classes.Message):
    __PB2_CLASS__ = maintenance_service_pb2.ListMaintenancesResponse
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.maintenance.v1alpha1.ListMaintenancesResponse",maintenance_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        items: "abc.Iterable[Maintenance]|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(items, unset.UnsetType):
            self.items = items
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "items",
        ]
    
    @builtins.property
    def items(self) -> "abc.MutableSequence[Maintenance]":
        """
        List of maintenance operations.
        """
        
        return super()._get_field("items", explicit_presence=False,
        wrap=pb_classes.Repeated.with_wrap(Maintenance,None,None),
        )
    @items.setter
    def items(self, value: "abc.Iterable[Maintenance]|None") -> None:
        return super()._set_field("items",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "items":"items",
    }
    
class GetMaintenanceRequest(pb_classes.Message):
    __PB2_CLASS__ = maintenance_service_pb2.GetMaintenanceRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.maintenance.v1alpha1.GetMaintenanceRequest",maintenance_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        """
        ID of the maintenance operation to return.
        To get the maintenance ID use a [MaintenanceService.List] request.
        """
        
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
    }
    
class UpdateMaintenanceRequest(pb_classes.Message):
    __PB2_CLASS__ = maintenance_service_pb2.UpdateMaintenanceRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.maintenance.v1alpha1.UpdateMaintenanceRequest",maintenance_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "MaintenanceSpec|maintenance_pb2.MaintenanceSpec|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        """
        Metadata associated with the maintenance operation.
        Must include ID of the maintenance operation to update.
        """
        
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "MaintenanceSpec":
        """
        Updated specification for the maintenance operation.
        """
        
        return super()._get_field("spec", explicit_presence=False,
        wrap=MaintenanceSpec,
        )
    @spec.setter
    def spec(self, value: "MaintenanceSpec|maintenance_pb2.MaintenanceSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
    }
    
class UpdateMaintenanceResponse(pb_classes.Message):
    __PB2_CLASS__ = maintenance_service_pb2.UpdateMaintenanceResponse
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.maintenance.v1alpha1.UpdateMaintenanceResponse",maintenance_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        maintenance: "Maintenance|maintenance_pb2.Maintenance|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(maintenance, unset.UnsetType):
            self.maintenance = maintenance
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "maintenance",
        ]
    
    @builtins.property
    def maintenance(self) -> "Maintenance":
        """
        Updated maintenance operation.
        """
        
        return super()._get_field("maintenance", explicit_presence=False,
        wrap=Maintenance,
        )
    @maintenance.setter
    def maintenance(self, value: "Maintenance|maintenance_pb2.Maintenance|None") -> None:
        return super()._set_field("maintenance",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "maintenance":"maintenance",
    }
    

class MaintenanceServiceClient(client.Client):
    """
    A set of methods for managing maintenance operations.
    
    This class provides the client methods for the ``.nebius.maintenance.v1alpha1.MaintenanceService`` service.
    
    Each method constructs a :class:`nebius.aio.request.Request` object
    that represents the in-flight RPC. The request can be awaited (async)
    or waited synchronously using its ``.wait()`` helpers.
    
    The request methods accept various parameters to configure metadata,
    timeouts, authorization, and retries. See individual method docstrings
    for details.
    
    :cvar __service_name__: The full protobuf service name.
    """
    
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.ServiceDescriptor](".nebius.maintenance.v1alpha1.MaintenanceService",maintenance_service_pb2.DESCRIPTOR,descriptor_1.ServiceDescriptor)
    """The protobuf service descriptor extraction function."""
    __service_name__ = ".nebius.maintenance.v1alpha1.MaintenanceService"
    
    def get(self,
        request: "GetMaintenanceRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request.Request["GetMaintenanceRequest","Maintenance"]:
        """
        Returns the specified maintenance operation.
        To get the list of upcoming and past maintenance operations, make a [List] request.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.maintenance.v1alpha1.GetMaintenanceRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.maintenance.v1alpha1.Maintenance`.
        """
        
        return super().request(
            method="Get",
            request=request,
            result_pb2_class=maintenance_pb2.Maintenance,
            result_wrapper=pb_classes.simple_wrapper(Maintenance),
            **kwargs,
        )
    
    def list(self,
        request: "ListMaintenancesRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request.Request["ListMaintenancesRequest","ListMaintenancesResponse"]:
        """
        Retrieves the list of maintenance operations that belong
        to the specified container.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.maintenance.v1alpha1.ListMaintenancesRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.maintenance.v1alpha1.ListMaintenancesResponse`.
        """
        
        return super().request(
            method="List",
            request=request,
            result_pb2_class=maintenance_service_pb2.ListMaintenancesResponse,
            result_wrapper=pb_classes.simple_wrapper(ListMaintenancesResponse),
            **kwargs,
        )
    
    def update(self,
        request: "UpdateMaintenanceRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request.Request["UpdateMaintenanceRequest","UpdateMaintenanceResponse"]:
        """
        Updates the specified maintenance operation.
        
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.maintenance.v1alpha1.UpdateMaintenanceRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.maintenance.v1alpha1.UpdateMaintenanceResponse`.
        """
        
        return super().request(
            method="Update",
            request=request,
            result_pb2_class=maintenance_service_pb2.UpdateMaintenanceResponse,
            result_wrapper=pb_classes.simple_wrapper(UpdateMaintenanceResponse),
            **kwargs,
        )
    

__all__ = [
    #@ local import names here @#
    "State",
    "Maintenance",
    "MaintenanceSpec",
    "MaintenanceStatus",
    "Resource",
    "ListMaintenancesRequest",
    "ListMaintenancesResponse",
    "GetMaintenanceRequest",
    "UpdateMaintenanceRequest",
    "UpdateMaintenanceResponse",
    "MaintenanceServiceClient",
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/mk8s/v1alpha1/progress_data.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(nebius/mk8s/v1alpha1/progress_data.proto\x12\x14nebius.mk8s.v1alpha1\x1a\x18nebius/annotations.proto\"I\n\x0cProgressData\x12\x39\n\x08problems\x18\x01 \x03(\x0b\x32\x1d.nebius.mk8s.v1alpha1.ProblemR\x08problems\"9\n\x07Problem\x12\x14\n\x05stage\x18\x01 \x01(\tR\x05stage\x12\x18\n\x07message\x18\x02 \x01(\tR\x07messageB{\n\x1b\x61i.nebius.pub.mk8s.v1alpha1B\x11ProgressDataProtoP\x01Z2github.com/nebius/gosdk/proto/nebius/mk8s/v1alpha1\xb8\x01\x01\xd2J\x0f\x12\rmigrate to v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.mk8s.v1alpha1.progress_data_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\033ai.nebius.pub.mk8s.v1alpha1B\021ProgressDataProtoP\001Z2github.com/nebius/gosdk/proto/nebius/mk8s/v1alpha1\270\001\001\322J\017\022\rmigrate to v1'
  _globals['_PROGRESSDATA']._serialized_start=92
  _globals['_PROGRESSDATA']._serialized_end=165
  _globals['_PROBLEM']._serialized_start=167
  _globals['_PROBLEM']._serialized_end=224
# @@protoc_insertion_point(module_scope)



import io
from collections.abc import Sequence
from keyword import iskeyword, issoftkeyword
from types import TracebackType
from typing import Any, Literal, TextIO

GENERATED_NOTE = """
Generated by the nebius.base.protos.compiler.  DO NOT EDIT!
"""


class GenerationError(Exception):
    pass


class ImportPath:
    def __init__(self, import_path: str, suggest_name: str | None = None) -> None:
        self.import_path = import_path
        self.suggest_name = suggest_name

    def __eq__(self, value: object) -> bool:
        if isinstance(value, ImportPath) and value.import_path == self.import_path:
            return True
        if isinstance(value, str) and value == self.import_path:
            return True
        return False


class ImportedSymbol:
    def __init__(self, name: str, import_path: ImportPath | str) -> None:
        self.name = name
        self.import_path = (
            import_path
            if isinstance(import_path, ImportPath)
            else ImportPath(import_path)
        )

    def __eq__(self, value: object) -> bool:
        if (
            isinstance(value, ImportedSymbol)
            and value.import_path == self.import_path
            and value.name == self.name
        ):
            return True
        return False

    def as_type_hint(self) -> "TypeHint":
        return TypeHint(self.name, self.import_path)


class TypeHint(ImportedSymbol):
    def as_symbol(self) -> ImportedSymbol:
        return ImportedSymbol(self.name, self.import_path)


Token = str | TypeHint | ImportedSymbol | ImportPath | Any


class PyGenFile:
    def __init__(
        self,
        import_path: str | ImportPath,
        indent_sequence: str | None = None,
        shebang_command: str | None = None,
        generated_note: str | None = None,
        used_names: Sequence[str] | None = None,
        docstring: str | None = None,
    ) -> None:
        self._shebang_command = shebang_command
        self._generated_note = (
            generated_note if generated_note is not None else GENERATED_NOTE
        )
        self._import_path = (
            import_path if isinstance(import_path, str) else import_path.import_path
        )
        self._used_names = set(used_names) if used_names is not None else set[str]()
        self._main_code = io.StringIO("")
        self._imports = dict[str, ImportPath]()
        self._imports_reversed = dict[str, str]()
        self._indent_sequence = (
            indent_sequence if indent_sequence is not None else "    "
        )
        self._indent = 0
        self._finalized = False
        self._docstring = docstring

    def is_local(self, import_path: ImportPath | str) -> bool:
        import_path_str = (
            import_path.import_path
            if isinstance(import_path, ImportPath)
            else import_path
        )
        return import_path_str == self._import_path

    def append_used_names(self, used_names: Sequence[str]) -> None:
        self._used_names = self._used_names.union(used_names)

    def suggest_name(
        self,
        suggestion: str,
        additional_names: Sequence[str] | None = None,
    ) -> str:
        ans = set(additional_names) if additional_names is not None else set[str]()

        def is_reserved(s: str) -> bool:
            return (
                s in self._imports_reversed
                or s in self._used_names
                or s in ans
                or iskeyword(s)
                or issoftkeyword(s)
            )

        if is_reserved(suggestion):
            inc = 1
            while True:
                new_suggest_name = f"{suggestion}_{inc}"
                if not is_reserved(new_suggest_name):
                    break
                inc += 1
            return new_suggest_name
        else:
            return suggestion

    def add_import(
        self,
        import_path: ImportPath | str,
        suggest_name: str | None = None,
    ) -> ImportPath:
        if self._finalized:
            GenerationError("can't add imports to finalized file")
        import_path_str = (
            import_path.import_path
            if isinstance(import_path, ImportPath)
            else import_path
        )
        if self.is_local(import_path):
            return ImportPath(import_path_str)
        if import_path_str in self._imports:
            return self._imports[import_path_str]
        if suggest_name is None and isinstance(import_path, ImportPath):
            suggest_name = import_path.suggest_name
        if suggest_name is None:
            suggest_name = import_path_str.split(".")[-1]
        if not isinstance(import_path, ImportPath):
            import_path = ImportPath(import_path, suggest_name)

        suggest_name = self.suggest_name(suggest_name)

        import_path = ImportPath(import_path.import_path, suggest_name)
        self._imports[import_path.import_path] = import_path
        self._imports_reversed[suggest_name] = import_path.import_path
        return import_path

    def set_docstring(self, docstring: str) -> None:
        self._docstring = docstring

    def indent(self) -> None:
        if self._finalized:
            GenerationError("can't indent finalized file")
        self._indent += 1

    def unindent(self) -> None:
        if self._indent <= 0:
            raise GenerationError("can't unindent more")
        self._indent -= 1

    def __enter__(self) -> "PyGenFile":
        self.indent()
        return self

    def __exit__(
        self,
        type_: type[BaseException] | None,
        value: BaseException | None,
        traceback: TracebackType | None,
    ) -> Literal[False]:
        self.unindent()
        return False

    def p(
        self,
        *args: Token,
        noindent: bool = False,
        add_eol: bool = True,
    ) -> None:
        if self._finalized:
            GenerationError("can't add lines to finalized file")
        if not noindent:
            self._main_code.write("".join([self._indent_sequence] * self._indent))
        for arg in args:
            if isinstance(arg, TypeHint):
                if self.is_local(arg.import_path):
                    self._main_code.write('"')
                    self._main_code.write(arg.name)
                    self._main_code.write('"')
                else:
                    import_path = self.add_import(arg.import_path)
                    self._main_code.write(str(import_path.suggest_name))
                    self._main_code.write(".")
                    self._main_code.write(arg.name)
            elif isinstance(arg, ImportedSymbol):
                if self.is_local(arg.import_path):
                    self._main_code.write(arg.name)
                else:
                    import_path = self.add_import(arg.import_path)
                    self._main_code.write(str(import_path.suggest_name))
                    self._main_code.write(".")
                    self._main_code.write(arg.name)
            elif isinstance(arg, ImportPath):
                if self.is_local(arg):
                    raise GenerationError(
                        f"can't add import path {arg.import_path} as an import to"
                        " itself"
                    )
                import_path = self.add_import(arg.import_path)
                self._main_code.write(str(import_path.suggest_name))
            else:
                self._main_code.write(f"{arg}")
        if add_eol:
            self._main_code.write("\n")

    def dump(self, writer: TextIO) -> None:
        self._finalized = True
        if self._shebang_command is not None:
            writer.write("#!")
            writer.write(self._shebang_command)
            writer.write("\n")
        gen_note = self._generated_note.split("\n")
        for line in gen_note:
            writer.write("# ")
            writer.write(line)
            writer.write("\n")
        writer.write("\n")
        if self._docstring is not None:
            writer.write('"""')
            writer.write(self._docstring)
            writer.write('"""\n\n')
        sorted_imports = sorted(
            self._imports.values(),
            key=lambda x: x.import_path,
        )
        for _imp in sorted_imports:
            writer.write("import ")
            writer.write(_imp.import_path)
            writer.write(" as ")
            writer.write(str(_imp.suggest_name))
            writer.write("\n")
        writer.write("#@ local imports here @#")
        writer.write("\n")
        writer.write("\n")
        writer.write(self._main_code.getvalue())
        writer.flush()

    def dumps(self) -> str:
        ret = io.StringIO("")
        self.dump(ret)
        return ret.getvalue()



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Record(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: RecordSpec
    status: RecordStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[RecordSpec, _Mapping]] = ..., status: _Optional[_Union[RecordStatus, _Mapping]] = ...) -> None: ...

class RecordSpec(_message.Message):
    __slots__ = ["relative_name", "type", "ttl", "data", "deletion_protection"]
    class RecordType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        RECORD_TYPE_UNSPECIFIED: _ClassVar[RecordSpec.RecordType]
        A: _ClassVar[RecordSpec.RecordType]
        AAAA: _ClassVar[RecordSpec.RecordType]
        PTR: _ClassVar[RecordSpec.RecordType]
        CNAME: _ClassVar[RecordSpec.RecordType]
        MX: _ClassVar[RecordSpec.RecordType]
        TXT: _ClassVar[RecordSpec.RecordType]
        SRV: _ClassVar[RecordSpec.RecordType]
        NS: _ClassVar[RecordSpec.RecordType]
        SOA: _ClassVar[RecordSpec.RecordType]
        CAA: _ClassVar[RecordSpec.RecordType]
        SVCB: _ClassVar[RecordSpec.RecordType]
        HTTPS: _ClassVar[RecordSpec.RecordType]
    RECORD_TYPE_UNSPECIFIED: RecordSpec.RecordType
    A: RecordSpec.RecordType
    AAAA: RecordSpec.RecordType
    PTR: RecordSpec.RecordType
    CNAME: RecordSpec.RecordType
    MX: RecordSpec.RecordType
    TXT: RecordSpec.RecordType
    SRV: RecordSpec.RecordType
    NS: RecordSpec.RecordType
    SOA: RecordSpec.RecordType
    CAA: RecordSpec.RecordType
    SVCB: RecordSpec.RecordType
    HTTPS: RecordSpec.RecordType
    RELATIVE_NAME_FIELD_NUMBER: _ClassVar[int]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    TTL_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    DELETION_PROTECTION_FIELD_NUMBER: _ClassVar[int]
    relative_name: str
    type: RecordSpec.RecordType
    ttl: int
    data: str
    deletion_protection: bool
    def __init__(self, relative_name: _Optional[str] = ..., type: _Optional[_Union[RecordSpec.RecordType, str]] = ..., ttl: _Optional[int] = ..., data: _Optional[str] = ..., deletion_protection: bool = ...) -> None: ...

class RecordStatus(_message.Message):
    __slots__ = ["zone_domain_name", "effective_fqdn", "reconciling"]
    ZONE_DOMAIN_NAME_FIELD_NUMBER: _ClassVar[int]
    EFFECTIVE_FQDN_FIELD_NUMBER: _ClassVar[int]
    RECONCILING_FIELD_NUMBER: _ClassVar[int]
    zone_domain_name: str
    effective_fqdn: str
    reconciling: bool
    def __init__(self, zone_domain_name: _Optional[str] = ..., effective_fqdn: _Optional[str] = ..., reconciling: bool = ...) -> None: ...

class GetRecordRequest(_message.Message):
    __slots__ = ["id", "resource_version"]
    ID_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_VERSION_FIELD_NUMBER: _ClassVar[int]
    id: str
    resource_version: int
    def __init__(self, id: _Optional[str] = ..., resource_version: _Optional[int] = ...) -> None: ...

class ListRecordsRequest(_message.Message):
    __slots__ = ["parent_id", "page_size", "page_token"]
    PARENT_ID_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    parent_id: str
    page_size: int
    page_token: str
    def __init__(self, parent_id: _Optional[str] = ..., page_size: _Optional[int] = ..., page_token: _Optional[str] = ...) -> None: ...

class ListRecordsResponse(_message.Message):
    __slots__ = ["items", "next_page_token"]
    ITEMS_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    items: _containers.RepeatedCompositeFieldContainer[Record]
    next_page_token: str
    def __init__(self, items: _Optional[_Iterable[_Union[Record, _Mapping]]] = ..., next_page_token: _Optional[str] = ...) -> None: ...



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as _metadata_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class GpuCluster(_message.Message):
    __slots__ = ["metadata", "spec", "status"]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    metadata: _metadata_pb2.ResourceMetadata
    spec: GpuClusterSpec
    status: GpuClusterStatus
    def __init__(self, metadata: _Optional[_Union[_metadata_pb2.ResourceMetadata, _Mapping]] = ..., spec: _Optional[_Union[GpuClusterSpec, _Mapping]] = ..., status: _Optional[_Union[GpuClusterStatus, _Mapping]] = ...) -> None: ...

class GpuClusterSpec(_message.Message):
    __slots__ = ["infiniband_fabric"]
    INFINIBAND_FABRIC_FIELD_NUMBER: _ClassVar[int]
    infiniband_fabric: str
    def __init__(self, infiniband_fabric: _Optional[str] = ...) -> None: ...

class GpuClusterStatus(_message.Message):
    __slots__ = ["instances", "reconciling", "infiniband_topology_path"]
    INSTANCES_FIELD_NUMBER: _ClassVar[int]
    RECONCILING_FIELD_NUMBER: _ClassVar[int]
    INFINIBAND_TOPOLOGY_PATH_FIELD_NUMBER: _ClassVar[int]
    instances: _containers.RepeatedScalarFieldContainer[str]
    reconciling: bool
    infiniband_topology_path: GpuClusterStatusInfinibandTopologyPath
    def __init__(self, instances: _Optional[_Iterable[str]] = ..., reconciling: bool = ..., infiniband_topology_path: _Optional[_Union[GpuClusterStatusInfinibandTopologyPath, _Mapping]] = ...) -> None: ...

class GpuClusterStatusInfinibandTopologyPath(_message.Message):
    __slots__ = ["instances"]
    INSTANCES_FIELD_NUMBER: _ClassVar[int]
    instances: _containers.RepeatedCompositeFieldContainer[GpuClusterStatusInfinibandTopologyPathInstance]
    def __init__(self, instances: _Optional[_Iterable[_Union[GpuClusterStatusInfinibandTopologyPathInstance, _Mapping]]] = ...) -> None: ...

class GpuClusterStatusInfinibandTopologyPathInstance(_message.Message):
    __slots__ = ["instance_id", "path"]
    INSTANCE_ID_FIELD_NUMBER: _ClassVar[int]
    PATH_FIELD_NUMBER: _ClassVar[int]
    instance_id: str
    path: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, instance_id: _Optional[str] = ..., path: _Optional[_Iterable[str]] = ...) -> None: ...



# 
# Generated by the nebius.base.protos.compiler.  DO NOT EDIT!
# 

"""Auto-generated Nebius SDK package for ``nebius.applications.v1alpha1``"""

import builtins as builtins
import collections.abc as abc
import google.protobuf.descriptor as descriptor_1
import google.protobuf.message as message_1
import nebius.aio.client as client
import nebius.aio.operation as operation
import nebius.aio.request as request_1
import nebius.aio.request_kwargs as request_kwargs
import nebius.api.nebius.applications.v1alpha1.k8s_release_pb2 as k8s_release_pb2
import nebius.api.nebius.applications.v1alpha1.k8s_release_service_pb2 as k8s_release_service_pb2
import nebius.api.nebius.common.v1 as v1_1
import nebius.api.nebius.common.v1.metadata_pb2 as metadata_pb2
import nebius.api.nebius.common.v1.operation_pb2 as operation_pb2
import nebius.base.fieldmask_protobuf as fieldmask_protobuf
import nebius.base.protos.descriptor as descriptor
import nebius.base.protos.pb_classes as pb_classes
import nebius.base.protos.pb_enum as pb_enum
import nebius.base.protos.unset as unset
import typing_extensions as typing_extensions
#@ local imports here @#

# file: nebius/applications/v1alpha1/k8s_release.proto
class K8sRelease(pb_classes.Message):
    __PB2_CLASS__ = k8s_release_pb2.K8sRelease
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.applications.v1alpha1.K8sRelease",k8s_release_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "K8sReleaseSpec|k8s_release_pb2.K8sReleaseSpec|None|unset.UnsetType" = unset.Unset,
        status: "K8sReleaseStatus|k8s_release_pb2.K8sReleaseStatus|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
        if not isinstance(status, unset.UnsetType):
            self.status = status
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
            "status",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "K8sReleaseSpec":
        return super()._get_field("spec", explicit_presence=False,
        wrap=K8sReleaseSpec,
        )
    @spec.setter
    def spec(self, value: "K8sReleaseSpec|k8s_release_pb2.K8sReleaseSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    @builtins.property
    def status(self) -> "K8sReleaseStatus":
        return super()._get_field("status", explicit_presence=False,
        wrap=K8sReleaseStatus,
        )
    @status.setter
    def status(self, value: "K8sReleaseStatus|k8s_release_pb2.K8sReleaseStatus|None") -> None:
        return super()._set_field("status",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
        "status":"status",
    }
    
class K8sReleaseSpec(pb_classes.Message):
    __PB2_CLASS__ = k8s_release_pb2.K8sReleaseSpec
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.applications.v1alpha1.K8sReleaseSpec",k8s_release_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    class SetEntry(pb_classes.Message):
        __PB2_CLASS__ = k8s_release_pb2.K8sReleaseSpec.SetEntry
        __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.applications.v1alpha1.K8sReleaseSpec.SetEntry",k8s_release_pb2.DESCRIPTOR,descriptor_1.Descriptor)
        __mask_functions__ = {
        }
        
        def __init__(
            self,
            initial_message: message_1.Message|None = None,
            *,
            key: "builtins.str|None|unset.UnsetType" = unset.Unset,
            value: "builtins.str|None|unset.UnsetType" = unset.Unset,
        ) -> None:
            super().__init__(initial_message)
            if not isinstance(key, unset.UnsetType):
                self.key = key
            if not isinstance(value, unset.UnsetType):
                self.value = value
        
        def __dir__(self) ->abc.Iterable[builtins.str]:
            return [
                "key",
                "value",
            ]
        
        @builtins.property
        def key(self) -> "builtins.str":
            return super()._get_field("key", explicit_presence=False,
            )
        @key.setter
        def key(self, value: "builtins.str|None") -> None:
            return super()._set_field("key",value,explicit_presence=False,
            )
        
        @builtins.property
        def value(self) -> "builtins.str":
            return super()._get_field("value", explicit_presence=False,
            )
        @value.setter
        def value(self, value: "builtins.str|None") -> None:
            return super()._set_field("value",value,explicit_presence=False,
            )
        
        __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
            "key":"key",
            "value":"value",
        }
        
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        cluster_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        product_slug: "builtins.str|None|unset.UnsetType" = unset.Unset,
        namespace: "builtins.str|None|unset.UnsetType" = unset.Unset,
        application_name: "builtins.str|None|unset.UnsetType" = unset.Unset,
        values: "builtins.str|None|unset.UnsetType" = unset.Unset,
        set: "abc.Mapping[builtins.str,builtins.str]|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(cluster_id, unset.UnsetType):
            self.cluster_id = cluster_id
        if not isinstance(product_slug, unset.UnsetType):
            self.product_slug = product_slug
        if not isinstance(namespace, unset.UnsetType):
            self.namespace = namespace
        if not isinstance(application_name, unset.UnsetType):
            self.application_name = application_name
        if not isinstance(values, unset.UnsetType):
            self.values = values
        if not isinstance(set, unset.UnsetType):
            self.set = set
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "cluster_id",
            "product_slug",
            "namespace",
            "application_name",
            "values",
            "set",
            "SetEntry",
        ]
    
    @builtins.property
    def cluster_id(self) -> "builtins.str":
        return super()._get_field("cluster_id", explicit_presence=False,
        )
    @cluster_id.setter
    def cluster_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("cluster_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def product_slug(self) -> "builtins.str":
        return super()._get_field("product_slug", explicit_presence=False,
        )
    @product_slug.setter
    def product_slug(self, value: "builtins.str|None") -> None:
        return super()._set_field("product_slug",value,explicit_presence=False,
        )
    
    @builtins.property
    def namespace(self) -> "builtins.str":
        return super()._get_field("namespace", explicit_presence=False,
        )
    @namespace.setter
    def namespace(self, value: "builtins.str|None") -> None:
        return super()._set_field("namespace",value,explicit_presence=False,
        )
    
    @builtins.property
    def application_name(self) -> "builtins.str":
        return super()._get_field("application_name", explicit_presence=False,
        )
    @application_name.setter
    def application_name(self, value: "builtins.str|None") -> None:
        return super()._set_field("application_name",value,explicit_presence=False,
        )
    
    @builtins.property
    def values(self) -> "builtins.str":
        return super()._get_field("values", explicit_presence=False,
        )
    @values.setter
    def values(self, value: "builtins.str|None") -> None:
        return super()._set_field("values",value,explicit_presence=False,
        )
    
    @builtins.property
    def set(self) -> "abc.MutableMapping[builtins.str,builtins.str]":
        return super()._get_field("set", explicit_presence=False,
        wrap=pb_classes.Map,
        )
    @set.setter
    def set(self, value: "abc.Mapping[builtins.str,builtins.str]|None") -> None:
        return super()._set_field("set",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "cluster_id":"cluster_id",
        "product_slug":"product_slug",
        "namespace":"namespace",
        "application_name":"application_name",
        "values":"values",
        "set":"set",
        "SetEntry":"SetEntry",
    }
    
class K8sReleaseStatus(pb_classes.Message):
    __PB2_CLASS__ = k8s_release_pb2.K8sReleaseStatus
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.applications.v1alpha1.K8sReleaseStatus",k8s_release_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    class State(pb_enum.Enum):
        __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.EnumDescriptor](".nebius.applications.v1alpha1.K8sReleaseStatus.State",k8s_release_pb2.DESCRIPTOR,descriptor_1.EnumDescriptor)
        UNSPECIFIED = 0
        CREATED = 1
        RUNNING = 2
        DEPLOYED = 3
        FAILED = 4
        INSTALLING = 5
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        state: "K8sReleaseStatus.State|k8s_release_pb2.K8sReleaseStatus.State|None|unset.UnsetType" = unset.Unset,
        error_message: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(state, unset.UnsetType):
            self.state = state
        if not isinstance(error_message, unset.UnsetType):
            self.error_message = error_message
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "state",
            "error_message",
            "State",
        ]
    
    @builtins.property
    def state(self) -> "K8sReleaseStatus.State":
        return super()._get_field("state", explicit_presence=False,
        wrap=K8sReleaseStatus.State,
        )
    @state.setter
    def state(self, value: "K8sReleaseStatus.State|k8s_release_pb2.K8sReleaseStatus.State|None") -> None:
        return super()._set_field("state",value,explicit_presence=False,
        )
    
    @builtins.property
    def error_message(self) -> "builtins.str":
        return super()._get_field("error_message", explicit_presence=False,
        )
    @error_message.setter
    def error_message(self, value: "builtins.str|None") -> None:
        return super()._set_field("error_message",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "state":"state",
        "error_message":"error_message",
        "State":"State",
    }
    
# file: nebius/applications/v1alpha1/k8s_release_service.proto
class GetK8sReleaseRequest(pb_classes.Message):
    __PB2_CLASS__ = k8s_release_service_pb2.GetK8sReleaseRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.applications.v1alpha1.GetK8sReleaseRequest",k8s_release_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
    }
    
class ListK8sReleasesRequest(pb_classes.Message):
    __PB2_CLASS__ = k8s_release_service_pb2.ListK8sReleasesRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.applications.v1alpha1.ListK8sReleasesRequest",k8s_release_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        parent_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
        page_size: "builtins.int|None|unset.UnsetType" = unset.Unset,
        page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
        filter: "builtins.str|None|unset.UnsetType" = unset.Unset,
        cluster_id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(parent_id, unset.UnsetType):
            self.parent_id = parent_id
        if not isinstance(page_size, unset.UnsetType):
            self.page_size = page_size
        if not isinstance(page_token, unset.UnsetType):
            self.page_token = page_token
        if not isinstance(filter, unset.UnsetType):
            self.filter = filter
        if not isinstance(cluster_id, unset.UnsetType):
            self.cluster_id = cluster_id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "parent_id",
            "page_size",
            "page_token",
            "filter",
            "cluster_id",
        ]
    
    @builtins.property
    def parent_id(self) -> "builtins.str":
        return super()._get_field("parent_id", explicit_presence=False,
        )
    @parent_id.setter
    def parent_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("parent_id",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_size(self) -> "builtins.int":
        return super()._get_field("page_size", explicit_presence=False,
        )
    @page_size.setter
    def page_size(self, value: "builtins.int|None") -> None:
        return super()._set_field("page_size",value,explicit_presence=False,
        )
    
    @builtins.property
    def page_token(self) -> "builtins.str":
        return super()._get_field("page_token", explicit_presence=False,
        )
    @page_token.setter
    def page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("page_token",value,explicit_presence=False,
        )
    
    @builtins.property
    def filter(self) -> "builtins.str":
        return super()._get_field("filter", explicit_presence=False,
        )
    @filter.setter
    def filter(self, value: "builtins.str|None") -> None:
        return super()._set_field("filter",value,explicit_presence=False,
        )
    
    @builtins.property
    def cluster_id(self) -> "builtins.str":
        return super()._get_field("cluster_id", explicit_presence=False,
        )
    @cluster_id.setter
    def cluster_id(self, value: "builtins.str|None") -> None:
        return super()._set_field("cluster_id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "parent_id":"parent_id",
        "page_size":"page_size",
        "page_token":"page_token",
        "filter":"filter",
        "cluster_id":"cluster_id",
    }
    
class CreateK8sReleaseRequest(pb_classes.Message):
    __PB2_CLASS__ = k8s_release_service_pb2.CreateK8sReleaseRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.applications.v1alpha1.CreateK8sReleaseRequest",k8s_release_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "K8sReleaseSpec|k8s_release_pb2.K8sReleaseSpec|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "K8sReleaseSpec":
        return super()._get_field("spec", explicit_presence=False,
        wrap=K8sReleaseSpec,
        )
    @spec.setter
    def spec(self, value: "K8sReleaseSpec|k8s_release_pb2.K8sReleaseSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
    }
    
class UpdateK8sReleaseRequest(pb_classes.Message):
    __PB2_CLASS__ = k8s_release_service_pb2.UpdateK8sReleaseRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.applications.v1alpha1.UpdateK8sReleaseRequest",k8s_release_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        metadata: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None|unset.UnsetType" = unset.Unset,
        spec: "K8sReleaseSpec|k8s_release_pb2.K8sReleaseSpec|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(metadata, unset.UnsetType):
            self.metadata = metadata
        if not isinstance(spec, unset.UnsetType):
            self.spec = spec
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "metadata",
            "spec",
        ]
    
    @builtins.property
    def metadata(self) -> "v1_1.ResourceMetadata":
        return super()._get_field("metadata", explicit_presence=False,
        wrap=v1_1.ResourceMetadata,
        )
    @metadata.setter
    def metadata(self, value: "v1_1.ResourceMetadata|metadata_pb2.ResourceMetadata|None") -> None:
        return super()._set_field("metadata",value,explicit_presence=False,
        )
    
    @builtins.property
    def spec(self) -> "K8sReleaseSpec":
        return super()._get_field("spec", explicit_presence=False,
        wrap=K8sReleaseSpec,
        )
    @spec.setter
    def spec(self, value: "K8sReleaseSpec|k8s_release_pb2.K8sReleaseSpec|None") -> None:
        return super()._set_field("spec",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "metadata":"metadata",
        "spec":"spec",
    }
    
class DeleteK8sReleaseRequest(pb_classes.Message):
    __PB2_CLASS__ = k8s_release_service_pb2.DeleteK8sReleaseRequest
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.applications.v1alpha1.DeleteK8sReleaseRequest",k8s_release_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        id: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(id, unset.UnsetType):
            self.id = id
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "id",
        ]
    
    @builtins.property
    def id(self) -> "builtins.str":
        return super()._get_field("id", explicit_presence=False,
        )
    @id.setter
    def id(self, value: "builtins.str|None") -> None:
        return super()._set_field("id",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "id":"id",
    }
    
class ListK8sReleasesResponse(pb_classes.Message):
    __PB2_CLASS__ = k8s_release_service_pb2.ListK8sReleasesResponse
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.Descriptor](".nebius.applications.v1alpha1.ListK8sReleasesResponse",k8s_release_service_pb2.DESCRIPTOR,descriptor_1.Descriptor)
    __mask_functions__ = {
    }
    
    def __init__(
        self,
        initial_message: message_1.Message|None = None,
        *,
        items: "abc.Iterable[K8sRelease]|None|unset.UnsetType" = unset.Unset,
        next_page_token: "builtins.str|None|unset.UnsetType" = unset.Unset,
    ) -> None:
        super().__init__(initial_message)
        if not isinstance(items, unset.UnsetType):
            self.items = items
        if not isinstance(next_page_token, unset.UnsetType):
            self.next_page_token = next_page_token
    
    def __dir__(self) ->abc.Iterable[builtins.str]:
        return [
            "items",
            "next_page_token",
        ]
    
    @builtins.property
    def items(self) -> "abc.MutableSequence[K8sRelease]":
        return super()._get_field("items", explicit_presence=False,
        wrap=pb_classes.Repeated.with_wrap(K8sRelease,None,None),
        )
    @items.setter
    def items(self, value: "abc.Iterable[K8sRelease]|None") -> None:
        return super()._set_field("items",value,explicit_presence=False,
        )
    
    @builtins.property
    def next_page_token(self) -> "builtins.str":
        return super()._get_field("next_page_token", explicit_presence=False,
        )
    @next_page_token.setter
    def next_page_token(self, value: "builtins.str|None") -> None:
        return super()._set_field("next_page_token",value,explicit_presence=False,
        )
    
    __PY_TO_PB2__: builtins.dict[builtins.str,builtins.str] = {
        "items":"items",
        "next_page_token":"next_page_token",
    }
    

class K8sReleaseServiceClient(client.ClientWithOperations[v1_1.Operation,v1_1.OperationServiceClient]):
    """
    This class provides the client methods for the ``.nebius.applications.v1alpha1.K8sReleaseService`` service.
    
    Each method constructs a :class:`nebius.aio.request.Request` object
    that represents the in-flight RPC. The request can be awaited (async)
    or waited synchronously using its ``.wait()`` helpers.
    
    The request methods accept various parameters to configure metadata,
    timeouts, authorization, and retries. See individual method docstrings
    for details.
    
    :cvar __service_name__: The full protobuf service name.
    """
    
    __PB2_DESCRIPTOR__ = descriptor.DescriptorWrap[descriptor_1.ServiceDescriptor](".nebius.applications.v1alpha1.K8sReleaseService",k8s_release_service_pb2.DESCRIPTOR,descriptor_1.ServiceDescriptor)
    """The protobuf service descriptor extraction function."""
    __service_name__ = ".nebius.applications.v1alpha1.K8sReleaseService"
    __operation_type__ = v1_1.Operation
    __operation_service_class__ = v1_1.OperationServiceClient
    __operation_source_method__ = "Create"
    """The method name that can be used to fetch the address channel for the operation."""
    
    def get(self,
        request: "GetK8sReleaseRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["GetK8sReleaseRequest","K8sRelease"]:
        """
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.applications.v1alpha1.GetK8sReleaseRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.applications.v1alpha1.K8sRelease`.
        """
        
        return super().request(
            method="Get",
            request=request,
            result_pb2_class=k8s_release_pb2.K8sRelease,
            result_wrapper=pb_classes.simple_wrapper(K8sRelease),
            **kwargs,
        )
    
    def list(self,
        request: "ListK8sReleasesRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["ListK8sReleasesRequest","ListK8sReleasesResponse"]:
        """
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.applications.v1alpha1.ListK8sReleasesRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.applications.v1alpha1.ListK8sReleasesResponse`.
        """
        
        return super().request(
            method="List",
            request=request,
            result_pb2_class=k8s_release_service_pb2.ListK8sReleasesResponse,
            result_wrapper=pb_classes.simple_wrapper(ListK8sReleasesResponse),
            **kwargs,
        )
    
    def create(self,
        request: "CreateK8sReleaseRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["CreateK8sReleaseRequest","operation.Operation[v1_1.Operation]"]:
        """
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.applications.v1alpha1.CreateK8sReleaseRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1.Operation`.
        """
        
        return super().request(
            method="Create",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    
    def update(self,
        request: "UpdateK8sReleaseRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["UpdateK8sReleaseRequest","operation.Operation[v1_1.Operation]"]:
        """
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.applications.v1alpha1.UpdateK8sReleaseRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1.Operation`.
        """
        
        kwargs['metadata'] = fieldmask_protobuf.ensure_reset_mask_in_metadata(request, kwargs.get('metadata', None))
        return super().request(
            method="Update",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    
    def delete(self,
        request: "DeleteK8sReleaseRequest",
        **kwargs: typing_extensions.Unpack[request_kwargs.RequestKwargs]
    ) -> request_1.Request["DeleteK8sReleaseRequest","operation.Operation[v1_1.Operation]"]:
        """
        :param request: The request object to send.
        :type request: :class:`nebius.api.nebius.applications.v1alpha1.DeleteK8sReleaseRequest`
        
        Other parameters can be provided as keyword arguments in the
        ``**kwargs`` dictionary, including metadata, timeouts, and retries.
        See :class:`nebius.aio.request_kwargs.RequestKwargs` for details.
        
        :return: A :class:`nebius.aio.request.Request` object representing the
            in-flight RPC. It can be awaited (async) or waited
            synchronously using its ``.wait()`` helpers.
        :rtype: :class:`nebius.aio.request.Request` of
            :class:`nebius.api.nebius.common.v1.Operation`.
        """
        
        return super().request(
            method="Delete",
            request=request,
            result_pb2_class=operation_pb2.Operation,
            result_wrapper=operation.Operation,
            **kwargs,
        )
    

__all__ = [
    #@ local import names here @#
    "K8sRelease",
    "K8sReleaseSpec",
    "K8sReleaseStatus",
    "GetK8sReleaseRequest",
    "ListK8sReleasesRequest",
    "CreateK8sReleaseRequest",
    "UpdateK8sReleaseRequest",
    "DeleteK8sReleaseRequest",
    "ListK8sReleasesResponse",
    "K8sReleaseServiceClient",
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/compute/v1alpha1/disk_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.common.v1alpha1 import operation_pb2 as nebius_dot_common_dot_v1alpha1_dot_operation__pb2
from nebius.api.nebius.common.v1alpha1 import operation_service_pb2 as nebius_dot_common_dot_v1alpha1_dot_operation__service__pb2
from nebius.api.nebius.compute.v1alpha1 import disk_pb2 as nebius_dot_compute_dot_v1alpha1_dot_disk__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*nebius/compute/v1alpha1/disk_service.proto\x12\x17nebius.compute.v1alpha1\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a&nebius/common/v1alpha1/operation.proto\x1a.nebius/common/v1alpha1/operation_service.proto\x1a\"nebius/compute/v1alpha1/disk.proto\" \n\x0eGetDiskRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"\xd5\x01\n\x10ListDisksRequest\x12\x1b\n\tparent_id\x18\x01 \x01(\tR\x08parentId\x12\x1b\n\tpage_size\x18\x02 \x01(\x03R\x08pageSize\x12\x1d\n\npage_token\x18\x03 \x01(\tR\tpageToken\x12h\n\x06\x66ilter\x18\x04 \x01(\tBP\x18\x01\xd2JK\n\n2025-06-16\x12=it is not implemented, filtering could be done on client sideR\x06\x66ilter\"\x8a\x01\n\x11\x43reateDiskRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12\x35\n\x04spec\x18\x02 \x01(\x0b\x32!.nebius.compute.v1alpha1.DiskSpecR\x04spec\"\x8a\x01\n\x11UpdateDiskRequest\x12>\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataR\x08metadata\x12\x35\n\x04spec\x18\x02 \x01(\x0b\x32!.nebius.compute.v1alpha1.DiskSpecR\x04spec\"#\n\x11\x44\x65leteDiskRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"p\n\x11ListDisksResponse\x12\x33\n\x05items\x18\x01 \x03(\x0b\x32\x1d.nebius.compute.v1alpha1.DiskR\x05items\x12&\n\x0fnext_page_token\x18\x02 \x01(\tR\rnextPageToken2\xa6\x05\n\x0b\x44iskService\x12M\n\x03Get\x12\'.nebius.compute.v1alpha1.GetDiskRequest\x1a\x1d.nebius.compute.v1alpha1.Disk\x12N\n\tGetByName\x12\".nebius.common.v1.GetByNameRequest\x1a\x1d.nebius.compute.v1alpha1.Disk\x12]\n\x04List\x12).nebius.compute.v1alpha1.ListDisksRequest\x1a*.nebius.compute.v1alpha1.ListDisksResponse\x12W\n\x06\x43reate\x12*.nebius.compute.v1alpha1.CreateDiskRequest\x1a!.nebius.common.v1alpha1.Operation\x12W\n\x06Update\x12*.nebius.compute.v1alpha1.UpdateDiskRequest\x1a!.nebius.common.v1alpha1.Operation\x12W\n\x06\x44\x65lete\x12*.nebius.compute.v1alpha1.DeleteDiskRequest\x1a!.nebius.common.v1alpha1.Operation\x12\x7f\n\x16ListOperationsByParent\x12\x35.nebius.common.v1alpha1.ListOperationsByParentRequest\x1a..nebius.common.v1alpha1.ListOperationsResponse\x1a\r\x88\x02\x01\xbaJ\x07\x63omputeBn\n\x1e\x61i.nebius.pub.compute.v1alpha1B\x10\x44iskServiceProtoP\x01Z5github.com/nebius/gosdk/proto/nebius/compute/v1alpha1\xb8\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.compute.v1alpha1.disk_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\036ai.nebius.pub.compute.v1alpha1B\020DiskServiceProtoP\001Z5github.com/nebius/gosdk/proto/nebius/compute/v1alpha1\270\001\001'
  _LISTDISKSREQUEST.fields_by_name['filter']._options = None
  _LISTDISKSREQUEST.fields_by_name['filter']._serialized_options = b'\030\001\322JK\n\n2025-06-16\022=it is not implemented, filtering could be done on client side'
  _DISKSERVICE._options = None
  _DISKSERVICE._serialized_options = b'\210\002\001\272J\007compute'
  _globals['_GETDISKREQUEST']._serialized_start=254
  _globals['_GETDISKREQUEST']._serialized_end=286
  _globals['_LISTDISKSREQUEST']._serialized_start=289
  _globals['_LISTDISKSREQUEST']._serialized_end=502
  _globals['_CREATEDISKREQUEST']._serialized_start=505
  _globals['_CREATEDISKREQUEST']._serialized_end=643
  _globals['_UPDATEDISKREQUEST']._serialized_start=646
  _globals['_UPDATEDISKREQUEST']._serialized_end=784
  _globals['_DELETEDISKREQUEST']._serialized_start=786
  _globals['_DELETEDISKREQUEST']._serialized_end=821
  _globals['_LISTDISKSRESPONSE']._serialized_start=823
  _globals['_LISTDISKSRESPONSE']._serialized_end=935
  _globals['_DISKSERVICE']._serialized_start=938
  _globals['_DISKSERVICE']._serialized_end=1616
# @@protoc_insertion_point(module_scope)



from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Template(_message.Message):
    __slots__ = ["status", "spec"]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    status: TemplateStatus
    spec: TemplateSpec
    def __init__(self, status: _Optional[_Union[TemplateStatus, _Mapping]] = ..., spec: _Optional[_Union[TemplateSpec, _Mapping]] = ...) -> None: ...

class TemplateStatus(_message.Message):
    __slots__ = ["preset_details"]
    PRESET_DETAILS_FIELD_NUMBER: _ClassVar[int]
    preset_details: PresetDetails
    def __init__(self, preset_details: _Optional[_Union[PresetDetails, _Mapping]] = ...) -> None: ...

class TemplateSpec(_message.Message):
    __slots__ = ["resources", "hosts", "disk", "role"]
    RESOURCES_FIELD_NUMBER: _ClassVar[int]
    HOSTS_FIELD_NUMBER: _ClassVar[int]
    DISK_FIELD_NUMBER: _ClassVar[int]
    ROLE_FIELD_NUMBER: _ClassVar[int]
    resources: ResourcesSpec
    hosts: Host
    disk: Disk
    role: str
    def __init__(self, resources: _Optional[_Union[ResourcesSpec, _Mapping]] = ..., hosts: _Optional[_Union[Host, _Mapping]] = ..., disk: _Optional[_Union[Disk, _Mapping]] = ..., role: _Optional[str] = ...) -> None: ...

class ResourcesSpec(_message.Message):
    __slots__ = ["platform", "preset"]
    PLATFORM_FIELD_NUMBER: _ClassVar[int]
    PRESET_FIELD_NUMBER: _ClassVar[int]
    platform: str
    preset: str
    def __init__(self, platform: _Optional[str] = ..., preset: _Optional[str] = ...) -> None: ...

class PresetDetails(_message.Message):
    __slots__ = ["cpu_count", "memory_gibibytes", "gpu_count"]
    CPU_COUNT_FIELD_NUMBER: _ClassVar[int]
    MEMORY_GIBIBYTES_FIELD_NUMBER: _ClassVar[int]
    GPU_COUNT_FIELD_NUMBER: _ClassVar[int]
    cpu_count: int
    memory_gibibytes: int
    gpu_count: int
    def __init__(self, cpu_count: _Optional[int] = ..., memory_gibibytes: _Optional[int] = ..., gpu_count: _Optional[int] = ...) -> None: ...

class Range(_message.Message):
    __slots__ = ["min", "max", "step", "value"]
    MIN_FIELD_NUMBER: _ClassVar[int]
    MAX_FIELD_NUMBER: _ClassVar[int]
    STEP_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    min: int
    max: int
    step: int
    value: int
    def __init__(self, min: _Optional[int] = ..., max: _Optional[int] = ..., step: _Optional[int] = ..., value: _Optional[int] = ...) -> None: ...

class Disk(_message.Message):
    __slots__ = ["type", "size_gibibytes"]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    SIZE_GIBIBYTES_FIELD_NUMBER: _ClassVar[int]
    type: str
    size_gibibytes: Range
    def __init__(self, type: _Optional[str] = ..., size_gibibytes: _Optional[_Union[Range, _Mapping]] = ...) -> None: ...

class DiskSpec(_message.Message):
    __slots__ = ["type", "size_gibibytes"]
    TYPE_FIELD_NUMBER: _ClassVar[int]
    SIZE_GIBIBYTES_FIELD_NUMBER: _ClassVar[int]
    type: str
    size_gibibytes: int
    def __init__(self, type: _Optional[str] = ..., size_gibibytes: _Optional[int] = ...) -> None: ...

class Host(_message.Message):
    __slots__ = ["count"]
    COUNT_FIELD_NUMBER: _ClassVar[int]
    count: Range
    def __init__(self, count: _Optional[_Union[Range, _Mapping]] = ...) -> None: ...

class HostSpec(_message.Message):
    __slots__ = ["count"]
    COUNT_FIELD_NUMBER: _ClassVar[int]
    count: int
    def __init__(self, count: _Optional[int] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/iam/v1/container.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from nebius.api.buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from nebius.api.nebius import annotations_pb2 as nebius_dot_annotations__pb2
from nebius.api.nebius.common.v1 import metadata_pb2 as nebius_dot_common_dot_v1_dot_metadata__pb2
from nebius.api.nebius.iam.v1 import state_pb2 as nebius_dot_iam_dot_v1_dot_state__pb2
from nebius.api.nebius.iam.v1 import suspension_state_pb2 as nebius_dot_iam_dot_v1_dot_suspension__state__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dnebius/iam/v1/container.proto\x12\rnebius.iam.v1\x1a\x1b\x62uf/validate/validate.proto\x1a\x18nebius/annotations.proto\x1a\x1fnebius/common/v1/metadata.proto\x1a\x19nebius/iam/v1/state.proto\x1a$nebius/iam/v1/suspension_state.proto\"\xcb\x01\n\tContainer\x12\x46\n\x08metadata\x18\x01 \x01(\x0b\x32\".nebius.common.v1.ResourceMetadataB\x06\xbaH\x03\xc8\x01\x01R\x08metadata\x12\x38\n\x04spec\x18\x02 \x01(\x0b\x32\x1c.nebius.iam.v1.ContainerSpecB\x06\xbaH\x03\xc8\x01\x01R\x04spec\x12<\n\x06status\x18\x03 \x01(\x0b\x32\x1e.nebius.iam.v1.ContainerStatusB\x04\xbaJ\x01\x05R\x06status\"-\n\rContainerSpec\x12\x1c\n\x06region\x18\x01 \x01(\tB\x04\xbaJ\x01\x07R\x06region\"\xb3\x01\n\x0f\x43ontainerStatus\x12I\n\x10suspension_state\x18\x02 \x01(\x0e\x32\x1e.nebius.iam.v1.SuspensionStateR\x0fsuspensionState\x12=\n\x0f\x63ontainer_state\x18\x03 \x01(\x0e\x32\x14.nebius.iam.v1.StateR\x0e\x63ontainerState\x12\x16\n\x06region\x18\x04 \x01(\tR\x06regionBU\n\x14\x61i.nebius.pub.iam.v1B\x0e\x43ontainerProtoP\x01Z+github.com/nebius/gosdk/proto/nebius/iam/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.iam.v1.container_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024ai.nebius.pub.iam.v1B\016ContainerProtoP\001Z+github.com/nebius/gosdk/proto/nebius/iam/v1'
  _CONTAINER.fields_by_name['metadata']._options = None
  _CONTAINER.fields_by_name['metadata']._serialized_options = b'\272H\003\310\001\001'
  _CONTAINER.fields_by_name['spec']._options = None
  _CONTAINER.fields_by_name['spec']._serialized_options = b'\272H\003\310\001\001'
  _CONTAINER.fields_by_name['status']._options = None
  _CONTAINER.fields_by_name['status']._serialized_options = b'\272J\001\005'
  _CONTAINERSPEC.fields_by_name['region']._options = None
  _CONTAINERSPEC.fields_by_name['region']._serialized_options = b'\272J\001\007'
  _globals['_CONTAINER']._serialized_start=202
  _globals['_CONTAINER']._serialized_end=405
  _globals['_CONTAINERSPEC']._serialized_start=407
  _globals['_CONTAINERSPEC']._serialized_end=452
  _globals['_CONTAINERSTATUS']._serialized_start=455
  _globals['_CONTAINERSTATUS']._serialized_end=634
# @@protoc_insertion_point(module_scope)



DOMAIN: str = "api.nebius.cloud:443"
"""Default Nebius API endpoint."""
DEFAULT_CONFIG_DIR: str = "~/.nebius"
"""Default Nebius configuration directory."""
DEFAULT_CONFIG_FILE: str = "config.yaml"
"""Default Nebius configuration file name in the configuration directory."""
DEFAULT_CREDENTIALS_FILE: str = "credentials.yaml"
"""Default Nebius credentials file name in the configuration directory."""
TOKEN_ENV = "NEBIUS_IAM_TOKEN"  # noqa: S105
"""Environment variable name for an IAM token string."""
PROFILE_ENV = "NEBIUS_PROFILE"
"""Environment variable name for a Nebius profile string."""
ENDPOINT_ENV = "NEBIUS_ENDPOINT"
"""Environment variable name for a Nebius API endpoint string."""



from collections.abc import Iterable

from nebius.base.metadata import Metadata
from nebius.base.protos.pb_classes import Message

RESET_MASK_HEADER = "X-ResetMask"


def ensure_reset_mask_in_metadata(
    msg: Message,
    metadata: Iterable[tuple[str, str]] | None,
) -> Metadata:
    metadata = Metadata(metadata)

    if RESET_MASK_HEADER not in metadata:  # type:ignore[comparison-overlap]
        metadata[RESET_MASK_HEADER] = Message.get_full_update_reset_mask(msg).marshal()
    return metadata



from nebius.api.nebius import annotations_pb2 as _annotations_pb2
from nebius.api.buf.validate import validate_pb2 as _validate_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class Payload(_message.Message):
    __slots__ = ["key", "string_value", "binary_value"]
    KEY_FIELD_NUMBER: _ClassVar[int]
    STRING_VALUE_FIELD_NUMBER: _ClassVar[int]
    BINARY_VALUE_FIELD_NUMBER: _ClassVar[int]
    key: str
    string_value: str
    binary_value: bytes
    def __init__(self, key: _Optional[str] = ..., string_value: _Optional[str] = ..., binary_value: _Optional[bytes] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nebius/storage/v1/base.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1cnebius/storage/v1/base.proto\x12\x11nebius.storage.v1*T\n\x0cStorageClass\x12\x1d\n\x19STORAGE_CLASS_UNSPECIFIED\x10\x00\x12\x0c\n\x08STANDARD\x10\x01\x12\x17\n\x13\x45NHANCED_THROUGHPUT\x10\x02*_\n\x10VersioningPolicy\x12!\n\x1dVERSIONING_POLICY_UNSPECIFIED\x10\x00\x12\x0c\n\x08\x44ISABLED\x10\x01\x12\x0b\n\x07\x45NABLED\x10\x02\x12\r\n\tSUSPENDED\x10\x03\x42X\n\x18\x61i.nebius.pub.storage.v1B\tBaseProtoP\x01Z/github.com/nebius/gosdk/proto/nebius/storage/v1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'nebius.storage.v1.base_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030ai.nebius.pub.storage.v1B\tBaseProtoP\001Z/github.com/nebius/gosdk/proto/nebius/storage/v1'
  _globals['_STORAGECLASS']._serialized_start=51
  _globals['_STORAGECLASS']._serialized_end=135
  _globals['_VERSIONINGPOLICY']._serialized_start=137
  _globals['_VERSIONINGPOLICY']._serialized_end=232
# @@protoc_insertion_point(module_scope)
